import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames/bind';
import { DFPProvider, AdSlot } from 'cnyes-dfp';

import { newsItemType, adProfileType } from '../../utils/propTypes';
import styles from './SlidingNews.scss';

const cx = classNames.bind(styles);

const SRC_SET_SPEC = {
  '1x': 'm',
  '2x': 'l',
  '3x': 'xl',
};

class SlidingNews extends React.Component {
  static propTypes = {
    speed: PropTypes.number,
    duration: PropTypes.number,
    items: PropTypes.arrayOf(PropTypes.oneOfType([newsItemType, adProfileType])),
  };

  static defaultProps = {
    speed: 1000,
    duration: 3000,
    items: [],
  };

  state = {
    index: 0,
    animate: false,
  };

  constructor(props) {
    super(props);

    this.displayedItemsLength = props.items.reduce((length, item) => {
      return item.newsId ? length + 1 : length;
    }, 0);

    this.queue = [];
    this.timeoutId = null;
  }

  componentDidMount() {
    this.sliderElement.addEventListener('transitionend', this.slide);

    this.slide();
  }

  componentWillUnmount() {
    this.sliderElement.removeEventListener('transitionend', this.slide);

    clearTimeout(this.timeoutId);
  }

  setSliderElement = element => {
    this.sliderElement = element;
  };

  handleSlotRenderEnded = (provider, event) => {
    const adElement = document.getElementById(`${event.slot.getSlotElementId()}-container`);

    if (!event.isEmpty) {
      this.queue.push(() => {
        adElement.style.display = 'block';
        this.displayedItemsLength += 1;
      });
    }
  };

  slide = () => {
    const { index } = this.state;
    const { duration } = this.props;

    if (index === this.displayedItemsLength) {
      this.setState(
        {
          index: 0,
          animate: false,
        },
        () => {
          this.queue.forEach(func => func && func());
          this.queue = [];
        }
      );
    }

    this.timeoutId = setTimeout(() => {
      this.setState(prevState => ({
        index: prevState.index + 1,
        animate: true,
      }));
    }, duration);
  };

  render() {
    const { index, animate } = this.state;
    const { speed, items } = this.props;

    const sliderElementStyle = {
      transform: `translateX(${-index * 100}%)`,
    };

    if (animate) {
      sliderElementStyle.transition = `transform ${speed}ms`;
    }

    return (
      <DFPProvider onSlotRenderEnded={this.handleSlotRenderEnded} loadGPT={false}>
        <div className={cx('wrapper')}>
          <div className={cx('slider')} ref={this.setSliderElement} style={sliderElementStyle}>
            {[...items, items[0]].map((item, i) => {
              const { newsId, title, hasCoverPhoto, coverSrc, path } = item;

              if (path) {
                return <AdSlot profile={item} key={path} />;
              }

              const defaultSize = SRC_SET_SPEC['1x'];
              const defaultSrc = coverSrc[defaultSize] && coverSrc[defaultSize].src;

              const srcSet = [];

              Object.keys(SRC_SET_SPEC).forEach(x => {
                const size = SRC_SET_SPEC[x];
                const src = coverSrc[size] && coverSrc[size].src;

                if (src) {
                  srcSet.push(`${src} ${x}`);
                }
              });

              return (
                // eslint-disable-next-line react/no-array-index-key
                <div className={cx('news')} key={`${newsId}-${i}`}>
                  {Boolean(hasCoverPhoto) &&
                    coverSrc &&
                    defaultSrc && <img className={cx('cover')} src={defaultSrc} srcSet={srcSet.join(',')} alt="" />}
                  <div className={cx('title')}>{title}</div>
                </div>
              );
            })}
          </div>
        </div>
      </DFPProvider>
    );
  }
}

export default SlidingNews;
